package com.unissoft.beidou.demo;

import com.yang.protocolstack.ProtocolStack;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import com.yang.protocolstack.protocol.*;

import java.net.InetAddress;
import java.util.Arrays;
import java.util.Date;

/**
 * @author qingsong.han
 * @description: 延迟启动, 加载类
 * @create 2022-12-15 17:19
 */
@Slf4j
@Component
@Order(1)
public class BeiDouConnect implements ApplicationRunner {

    //1.获取配置ip port
    @Value("${customSocket.beidou.command.ip}")
    private String ip;
    @Value("${customSocket.beidou.command.port}")
    private int port;
    @Value("${customSocket.beidou.command.protocolType}")
    private int protocolType;

    // 全局定义
    private SocketClient socketClient;
    private ProtocolStack protocolStack;
    private int id;
    private int SubCount = 0;

    private Thread readThread = new Thread(new Runnable() {
        @Override
        public void run() {
            while (true) {
                Response response;
                if (protocolStack != null) {
                    response = protocolStack.getResponse();
                    if (response != null) {
                        switch (response.getMsgType()) {
                            case MsgType.BSI_PROTOCOL:
                                //2.1协议
                                log.info("波束强度: {}" + Arrays.toString(((BDBSIResponse) response).getPower()));
                                break;
                            case MsgType.ZDA_PROTOCOL:
                                //2.1协议
                                log.info("时间: {}" + ((BDZDAResponse) response).getTimeBJ());
                                break;
                            case MsgType.GLZK_PROTOCOL:
                                //4.0协议
                                log.info("波束强度: {}" + Arrays.toString(((BDGLZKResponse) response).getPower()));
                                break;
                            case MsgType.ICXX_PROTOCOL:
                                log.info(
                                        "北斗卡号: {}", ((BDICXXResponse) response).getId(),
                                        "通播地址: {}", ((BDICXXResponse) response).getBroadcastId(),
                                        "用户机特征: {}", ((BDICXXResponse) response).getFeature(),
                                        "服务频度: {}", ((BDICXXResponse) response).getServerFreq(),
                                        "通信长度: {}", ((BDICXXResponse) response).getMaxCommLength()
                                );
                                id = ((BDICXXResponse) response).getId();
                                log.info("Thread id: {}", id);
                                SubCount = ((BDICXXResponse) response).getNumber();
                                log.info("Thread SubCount: {}", SubCount);
                                break;
                            case MsgType.ICI_PROTOCOL:
                                log.info(
                                        "北斗卡号: {}", ((BDICIResponse) response).getLocalId(),
                                        "通播地址: {}", ((BDICIResponse) response).getBroadcastId(),
                                        "用户机特征: {}", ((BDICIResponse) response).getFeature(),
                                        "服务频度: {}", ((BDICIResponse) response).getServerFreq(),
                                        "通信长度: {}", ((BDICIResponse) response).getMaxCommLength()
                                );
                                id = ((BDICIResponse) response).getLocalId();
                                log.info("Thread id: {}", id);
                                break;
                            case MsgType.SJXX_PROTOCOL:
                                //4.0协议
                                log.info("时间: {}" + ((BDSJXXResponse) response).getTime());
                                break;
                            case MsgType.DWR_PROTOCOL:
                                log.info(String.format("定位位置：经度： %.06f 纬度： %.06f %n", ((BDDWRResponse) response).getLongitude(), ((BDDWRResponse) response).getLatitude()));
                                break;
                            case MsgType.DWXX_PROTOCOL:
                                log.info(String.format("定位位置：经度： %s 纬度： %s %n", ((BDDWXXResponse) response).getLongitude(), ((BDDWXXResponse) response).getLatitude()));
                                break;
                            case MsgType.FKI_PROTOCOL:
                                if (((BDFKIResponse) response).isCmdRes()) {
                                    log.info("申请成功，等待反馈！");
                                } else {
                                    log.info(String.format("申请失败，频度未到。等待时间：%d s%n", ((BDFKIResponse) response).getWaitTime()));
                                }
                                break;
                            case MsgType.FKXX_PROTOCOL:
                                if (((BDFKXXResponse) response).isCmdResult()) {
                                    log.info("申请成功，等待反馈！");
                                } else {
                                    log.info(String.format("申请失败，频度未到。等待时间：%d s%n", ((BDFKXXResponse) response).getWaitTime()));
                                }
                                break;
                            case MsgType.WAA_PROTOCOL:
                                log.info(String.format("收到位置报告：经度： %.06f 纬度： %.06f %n", ((BDWAAResponse) response).getLongitude(), ((BDWAAResponse) response).getLatitude()));
                                break;
                            case MsgType.TXR_PROTOCOL:
                                log.info(String.format("收到通信信息：内容： %s 发送方： %d 通信类型： %d %n", ((BDTXRResponse) response).getCommInfo(), ((BDTXRResponse) response).getSenderID(), ((BDTXRResponse) response).getCodeType()));
                                break;
                            case MsgType.TXXX_PROTOCOL:
                                if (((BDTXXXResponse) response).getType() == 3) {
                                    log.info(
                                            String.format("4.0协议通过通信发送接收位置报告，接收到的位置信息: 时间： %tT，经度： %s，纬度： %s，高程： %d%n", new Date(((BDTXXXResponse) response).getTime()),
                                                    ((BDTXXXResponse) response).getLongitude(), ((BDTXXXResponse) response).getLatitude(), ((BDTXXXResponse) response).getAltitude())
                                    );
                                } else {
                                    log.info(String.format("收到通信信息：内容： %s 发送方： %s 通信类型： %d %n", ((BDTXXXResponse) response).getContent(), ((BDTXXXResponse) response).getSendId(), ((BDTXXXResponse) response).getType()));
                                }
                                break;
                            case MsgType.ZJX_PROTOCOL: {//2.1协议
                                log.info("自检信息 IC状态: {}", ((BDZJXResponse) response).getCardState());
                                log.info("自检信息 IC状态: {}", ((BDZJXResponse) response).getCardState());
                                log.info("自检信息 入站状态: {}", ((BDZJXResponse) response).getInState());
                            }
                            break;
                        }
                    } else {
                        try {
                            Thread.sleep(1);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                } else {
                    try {
                        Thread.sleep(1);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    });


    //2.打开网口
    private void openSocket() {
        try {
            socketClient = new SocketClient(ip, port) {
                @Override
                public void onReceive(InetAddress addr, byte[] bytes) {
                    protocolStack.setBytes(bytes);
                }

                @Override
                public void onDisconnect(InetAddress addr) {
                    log.info("openSocket addr: {}", addr);
                }
            };
            protocolStack = new ProtocolStack();
            protocolStack.init(protocolType, false);
            socketClient.start();
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        if (!readThread.isAlive()) {
            readThread.start();
        }
    }

    // 关闭网口
    private void closeSocket() {
        protocolStack.stop();
        socketClient.stop();
        log.info("网口已关闭");
        protocolStack = null;
    }

    private void actionListener() throws Exception {
        // 打开网口
        log.info("打开网口中...");
        Thread.sleep(1);
        this.openSocket();
        log.info("网口已打开!");
        log.info("读卡申请中...");
        if (protocolStack != null) {
            //3.读卡
            Thread.sleep(1);
            socketClient.send(protocolStack.getDKSQ());
            Thread.sleep(1000);
            if (id == 0) {
                log.info("未读卡,在获取到卡号之后才能定位...");
            } else {
                //4.获取定位
                log.info("定位申请中...");
                socketClient.send(protocolStack.getDWSQ(String.valueOf(id), false, 0));

                //5.读取下属
                log.info("读子用户申请中...");
                Thread.sleep(1);
                byte icNo = 1;
                for (int i = 0; i < SubCount / 20; i++)
                    //默认百用户  千用户时需要设置为true
                    socketClient.send(protocolStack.getZYHDKSQ(icNo, id + "", false));

                //6.下属位置报告
                log.info("发送位置报告中...");
                Thread.sleep(1);
                if (protocolType == 0) {
                    //仅2.1协议
                    socketClient.send(protocolStack.getWZBG(String.valueOf(id), 0, null));
                } else {
                    socketClient.send(protocolStack.getWZXX40(String.valueOf(id), new Date().getTime(), 38.123d, 112.456d, 153, String.valueOf(id)));
                }
            }
        }

    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        actionListener();
    }

}
